package de.hftstuttgart.cityunits.model;

import java.math.BigInteger;

import javax.measure.Dimension;
import javax.measure.Quantity;
import javax.measure.Unit;
import javax.measure.quantity.Dimensionless;
import javax.measure.quantity.Mass;
import javax.measure.spi.SystemOfUnits;

import tech.units.indriya.AbstractSystemOfUnits;
import tech.units.indriya.AbstractUnit;
import tech.units.indriya.format.SimpleUnitFormat;
import tech.units.indriya.function.LogConverter;
import tech.units.indriya.function.MultiplyConverter;
import tech.units.indriya.unit.AlternateUnit;
import tech.units.indriya.unit.BaseUnit;
import tech.units.indriya.unit.TransformedUnit;
import tech.units.indriya.unit.UnitDimension;
import tech.units.indriya.unit.Units;

/**
 * See https://www.openhab.org/docs/concepts/units-of-measurement.html and
 * https://github.com/unitsofmeasurement/uom-demos/blob/master/domain/energy/src/main/java/tech/uom/demo/energy/
 */
public class UrbanSimulationUnits extends AbstractSystemOfUnits {
	private static final UrbanSimulationUnits INSTANCE = new UrbanSimulationUnits();

	@Override
	public String getName() {
		return getClass().getSimpleName();
	}

	public static final Unit<Mass> TON = addUnit(Units.KILOGRAM.multiply(1000));
	public static final Unit<CarbonEmission> KILOGRAM_CO2_EQUIVALENT = addUnit(
			AlternateUnit.of(Units.KILOGRAM, "kgCO₂eq").asType(CarbonEmission.class));
	public static final Unit<CarbonEmission> GRAM_CO2_EQUIVALENT = addUnit(KILOGRAM_CO2_EQUIVALENT.divide(1000));
	public static final Unit<CarbonEmission> TON_CO2_EQUIVALENT = addUnit(KILOGRAM_CO2_EQUIVALENT.multiply(1000));

	public static final Unit<Dimensionless> PARTS_PER_MILLION = addUnit(new TransformedUnit<>(AbstractUnit.ONE,
			MultiplyConverter.ofRational(BigInteger.ONE, BigInteger.valueOf(1000000))));
	public static final Unit<Dimensionless> DECIBEL = addUnit(AbstractUnit.ONE.transform(
			new LogConverter(10).inverse().concatenate(MultiplyConverter.ofRational(BigInteger.ONE, BigInteger.TEN))));
	public static final Unit<Intensity> IRRADIANCE = addUnit(
			new AlternateUnit<Intensity>(Units.WATT.divide(Units.SQUARE_METRE), "W/m²"));

	// To model costs I added monetary units quick and dirty as SI base units.
	// According to JavaDoc of AbstractUnit, monetary units should rather be
	// implemented in an extra type hierarchy below ComparableUnit.
	public final static Dimension MONEY_DIMENSION = UnitDimension.parse('M');
	public final static Unit<Euro> EURO = new BaseUnit<Euro>("€", MONEY_DIMENSION);
	public final static Unit<Dollar> DOLLAR = new BaseUnit<Dollar>("$", MONEY_DIMENSION);

	static {
		SimpleUnitFormat.getInstance().label(TON, "t");
		SimpleUnitFormat.getInstance().label(KILOGRAM_CO2_EQUIVALENT, "kgCO₂eq");
		SimpleUnitFormat.getInstance().label(GRAM_CO2_EQUIVALENT, "gCO₂eq");
		SimpleUnitFormat.getInstance().label(TON_CO2_EQUIVALENT, "tonCO₂eq");
		SimpleUnitFormat.getInstance().label(DECIBEL, "dB");
		SimpleUnitFormat.getInstance().label(PARTS_PER_MILLION, "ppm");
		SimpleUnitFormat.getInstance().label(EURO, "€");
		SimpleUnitFormat.getInstance().label(DOLLAR, "$");
	}

	/**
	 * Returns the unique instance of this class.
	 *
	 * @return the Units instance.
	 */
	public static SystemOfUnits getInstance() {
		return INSTANCE;
	}

	/**
	 * Adds a new unit not mapped to any specified quantity type.
	 *
	 * @param unit the unit being added.
	 * @return <code>unit</code>.
	 */
	private static <U extends Unit<?>> U addUnit(U unit) {
		INSTANCE.units.add(unit);
		return unit;
	}

	/**
	 * Adds a new unit and maps it to the specified quantity type.
	 *
	 * @param unit the unit being added.
	 * @param type the quantity type.
	 * @return <code>unit</code>.
	 */
	private static <U extends AbstractUnit<?>> U addUnit(U unit, Class<? extends Quantity<?>> type) {
		INSTANCE.units.add(unit);
		INSTANCE.quantityToUnit.put(type, unit);
		return unit;
	}
}
